@@ -1,13 +1,26 @@
 riskParityPortfolioDiagSigma <- function(Sigma, b = rep(1/nrow(Sigma), nrow(Sigma))) {
   w <- sqrt(b) / sqrt(diag(Sigma))
   w <- w / sum(w)
-  return (list(w = w, risk_contribution = as.vector(w * (Sigma %*% w))))
+  w_Sigmaw <- as.vector(w * (Sigma %*% w))
+  return (list(w = w, relative_risk_contribution = w_Sigmaw / sum(w_Sigmaw)))
 }
 
 
-riskParityPortfolioSCA <- function(Sigma, b = rep(1/nrow(Sigma), nrow(Sigma)),
-                                   mu = NULL, lmd_mu = 1e-4, lmd_var = 0,
+isFeasiblePortfolio <- function(w, Cmat, cvec, Dmat, dvec, tol = 1e-6) {
+  equality_feasibility <- all(abs(Cmat %*% w - cvec) < tol)
+  if (is.null(Dmat))
+    return (equality_feasibility)
+  inequality_feasibility <- all(Dmat %*% w - dvec <= tol)
+  return (equality_feasibility && inequality_feasibility)
+}
+
+
+riskParityPortfolioSCA <- function(Sigma, w0, b = rep(1/nrow(Sigma), nrow(Sigma)),
+                                   mu = NULL, lmd_mu = 0, lmd_var = 0,
                                    w_lb = rep(0, nrow(Sigma)), w_ub = rep(1, nrow(Sigma)),
+                                   Cmat = matrix(1, 1, nrow(Sigma)), cvec = c(1),
+                                   Dmat = rbind(diag(nrow(Sigma)), -diag(nrow(Sigma))),
+                                   dvec = c(w_ub, -w_lb),
                                    formulation = c("rc-over-b-double-index",
                                                    "rc-double-index",
                                                    "rc-over-var vs b",
@@ -16,33 +29,45 @@ riskParityPortfolioSCA <- function(Sigma, b = rep(1/nrow(Sigma), nrow(Sigma)),
                                                    "rc vs b-times-var",
                                                    "rc vs theta",
                                                    "rc-over-b vs theta"),
-                                   w0 = NULL, theta0 = NULL, gamma = .9, zeta = 1e-7,
-                                   tau = NULL, maxiter = 500, ftol = 1e-6, wtol = 1e-6) {
+                                   theta0 = NULL, gamma = .9, zeta = 1e-7,
+                                   tau = NULL, maxiter = 1000, ftol = 1e-8, wtol = .5e-6,
+                                   use_qp_solver = TRUE) {
   N <- nrow(Sigma)
-  if (is.null(w0)) w0 <- projectBudgetLineAndBox(riskParityPortfolioDiagSigma(Sigma, b)$w, w_lb, w_ub)
   formulation <- match.arg(formulation)
+  # define boolean cases for easier reading of code
+  has_mu <- !is.null(mu)
+  has_var <- lmd_var > 0
   has_theta <- grepl("theta", formulation)
   if (has_theta) {
     if (is.null(theta0)) {
       r0 <- w0 * (Sigma %*% w0)
-      if (formulation == "rc vs theta")
-        theta0 <- mean(r0)
-      else if (formulation == "rc-over-b vs theta")
-        theta0 <- mean(r0 / b)
-    }
+      theta0 <- switch(formulation,
+                       "rc vs theta" = mean(r0),
+                       "rc-over-b vs theta" = mean(r0 / b),
+                       stop("Oops... Formulation name contains theta but it is not supported."))
+      }
     w0 <- as.vector(c(w0, theta0))
   }
+  # parameters tau for SCA method
   if (is.null(tau))
     tau <- .05 * sum(diag(Sigma)) / (2*N)
+  tauI <- diag(rep(tau, length(w0)))
+  # packing linear constrains
   if (has_theta) {
-    Amat <- cbind(c(rep(1, N), 0), diag(rep(1, N+1)), -diag(c(rep(1, N), 0)))
-    bvec <- c(1, c(w_lb, 0), c(-w_ub, 0))
+    Cmat <- cbind(Cmat, 0)
+    Dmat <- cbind(Dmat, 0)
+  }
+  # check the type of constraints
+  has_only_equality_constraints <- all(w_lb == (-Inf), w_ub == Inf)
+  # initiliaze some variables depending on the type of solver
+  if (use_qp_solver) {
+    meq <- nrow(Cmat)
+    Amat <- t(rbind(Cmat, -Dmat))
+    bvec <- c(cvec, -dvec)
   } else {
-    Amat <- cbind(rep(1, N), diag(N), -diag(N))
-    bvec <- c(1, w_lb, -w_ub)
+    dual_lmd_0 <- dual_lmd_minus_1 <- rep(0, length(cvec))
+    dual_mu_0 <- dual_mu_minus_1 <- rep(0, length(dvec))
   }
-  meq <- 1
-
   switch(formulation,
          "rc-double-index" = {
            R <- R_rc_double_index
@@ -86,21 +111,19 @@ riskParityPortfolioSCA <- function(Sigma, b = rep(1/nrow(Sigma), nrow(Sigma)),
          },
          stop("formulation ", formulation, " is not included.")
   )
-  has_mu <- !is.null(mu)
   # compute and store objective function at the initial value
   wk <- w0
   fun_k <- R(wk, Sigma, b)
   if (has_mu) {
+    if (lmd_mu == 0) warning("The mean return vector has been given, but lmd_mu = 0.")
     if (has_theta) fun_k <- fun_k - lmd_mu * t(mu) %*% wk[1:N]
     else fun_k <- fun_k - lmd_mu * t(mu) %*% wk
   }
+  if (has_var) fun_k <- fun_k + lmd_var * t(wk) %*% Sigma %*% wk
   fun_seq <- c(fun_k)
   time_seq <- c(0)
 
-  if (has_theta) tauI <- diag(rep(tau, N + 1))
-  else tauI <- diag(rep(tau, N))
-  has_var <- lmd_var > 0
-
+  # SCA outer loop
   start_time <- proc.time()[3]
   for (k in 1:maxiter) {
     # auxiliary quantities
@@ -121,9 +144,21 @@ riskParityPortfolioSCA <- function(Sigma, b = rep(1/nrow(Sigma), nrow(Sigma)),
     if (has_mu)
       if (has_theta) qk <- qk - lmd_mu * c(mu, 0)
       else qk <- qk - lmd_mu * mu
-    # build and solve problem (39) as in Feng & Palomar TSP2015
-    w_hat <- quadprog::solve.QP(Qk, -qk, Amat = Amat, bvec = bvec,
-                                meq = meq)$solution
+    # solve problem (39) as in Feng & Palomar TSP2015
+    if (use_qp_solver) {
+      w_hat <- quadprog::solve.QP(Qk, -qk, Amat = Amat, bvec = bvec, meq = meq)$solution
+    } else if (has_only_equality_constraints) {
+      w_hat <- rpp_equality_constraints_iteration(Cmat, cvec, Qk, qk)
+    } else {
+      params <- rpp_eq_and_ineq_constraints_iteration(Cmat, cvec, Dmat, dvec, Qk, qk, wk,
+                                                      dual_mu_0, dual_mu_minus_1, dual_lmd_0,
+                                                      dual_lmd_minus_1, maxiter, wtol)
+      dual_mu_minus_1 <- params[[1]]
+      dual_mu_0 <- params[[2]]
+      dual_lmd_minus_1 <- params[[3]]
+      dual_lmd_0 <- params[[4]]
+      w_hat <- params[[5]]
+    }
     w_next <- wk + gamma * (w_hat - wk)
     # save objective function value and elapsed time
     time_seq <- c(time_seq, proc.time()[3] - start_time)
@@ -134,34 +169,36 @@ riskParityPortfolioSCA <- function(Sigma, b = rep(1/nrow(Sigma), nrow(Sigma)),
     if (has_var)
       if (has_theta) fun_next <- fun_next + lmd_var * (t(w_next[1:N]) %*% Sigma %*% w_next[1:N])
       else fun_next <- fun_next + lmd_var * (t(w_next) %*% Sigma %*% w_next)
-    fun_seq <- c(fun_seq, fun_next)
     # check convergence on parameters and objective function
     has_w_converged <- all((abs(w_next - wk) <= .5 * wtol * (abs(wk) + abs(w_next))))
-    has_fun_converged <- abs(fun_next - fun_k) <= .5 * ftol * (abs(fun_k) + abs(fun_next))
-    if (k > 1 && (has_w_converged && has_fun_converged)) break
+    has_fun_converged <- (abs(fun_next - fun_k) <= .5 * ftol * (abs(fun_k) + abs(fun_next)))
+    if (k > 1 && (has_w_converged || has_fun_converged)) break
     # update variables
     wk <- w_next
+    fun_seq <- c(fun_seq, fun_next)
     fun_k <- fun_next
     gamma <- gamma * (1 - zeta * gamma)
   }
 
   portfolio_results <- list()
-  portfolio_results$risk_parity <- fun_seq[length(fun_seq)]
+  portfolio_results$risk_concentration <- fun_seq[length(fun_seq)]
   if (!has_theta) {
     portfolio_results$w <- w_next
-    portfolio_results$risk_contribution <- as.vector(w_next * (Sigma %*% w_next))
+    w_Sigmaw <- as.vector(w_next * (Sigma %*% w_next))
+    portfolio_results$relative_risk_contribution <- w_Sigmaw / sum(w_Sigmaw)
   } else {
     portfolio_results$w <- w_next[1:N]
     portfolio_results$theta <- w_next[N+1]
-    portfolio_results$risk_contribution <- as.vector(w_next[1:N] * (Sigma %*% w_next[1:N]))
+    w_Sigmaw <- as.vector(w_next[1:N] * (Sigma %*% w_next[1:N]))
+    portfolio_results$relative_risk_contribution <- w_Sigmaw / sum(w_Sigmaw)
   }
   if (has_mu) {
     portfolio_results$mean_return <- t(mu) %*% portfolio_results$w
-    portfolio_results$risk_parity <- portfolio_results$risk_parity + lmd_mu * portfolio_results$mean_return
+    portfolio_results$risk_concentration <- portfolio_results$risk_concentration + lmd_mu * portfolio_results$mean_return
   }
   if (has_var) {
     portfolio_results$variance <- t(portfolio_results$w) %*% Sigma %*% portfolio_results$w
-    portfolio_results$risk_parity <- portfolio_results$risk_parity - lmd_var * portfolio_results$variance
+    portfolio_results$risk_concentration <- portfolio_results$risk_concentration - lmd_var * portfolio_results$variance
   }
   portfolio_results$obj_fun <- fun_seq
   portfolio_results$elapsed_time <- time_seq
@@ -170,184 +207,11 @@ riskParityPortfolioSCA <- function(Sigma, b = rep(1/nrow(Sigma), nrow(Sigma)),
 }
 
 
-riskParityPortfolioGenSolver <- function(Sigma, b = rep(1/nrow(Sigma), nrow(Sigma)),
-                                         mu = NULL, lmd_mu = 1e-4,
-                                         formulation = c("rc-over-b-double-index",
-                                                         "rc-double-index",
-                                                         "rc-over-var vs b",
-                                                         "rc-over-var",
-                                                         "rc-over-sd vs b-times-sd",
-                                                         "rc vs b-times-var",
-                                                         "rc vs theta",
-                                                         "rc-over-b vs theta"),
-                                         method = c("slsqp", "alabama"),
-                                         use_gradient = TRUE, w0 = NULL, theta0 = NULL,
-                                         maxiter = 500, ftol = 1e-6, wtol = 1e-6) {
-  N <- nrow(Sigma)
-  if (is.null(w0)) w0 <- riskParityPortfolioDiagSigma(Sigma, b)$w
-  formulation <- match.arg(formulation)
-  # set initial value for theta
-  has_theta <- grepl("theta", formulation)
-  if (has_theta) {
-    if (is.null(theta0)) {
-      r0 <- w0 * (Sigma %*% w0)
-      if (formulation == "rc vs theta")
-        theta0 <- mean(r0)
-      else if (formulation == "rc-over-b vs theta")
-        theta0 <- mean(r0 / b)
-    }
-    w0 <- as.vector(c(w0, theta0))
-  }
-  if (has_theta) {
-    budget <- function(w, ...) {
-      N <- length(w) - 1
-      return(sum(w[1:N]) - 1)
-    }
-    budget.jac <- function(w, ...) {
-      N <- length(w) - 1
-      return(cbind(matrix(1, 1, N), 0))
-    }
-    shortselling <- function(w, ...) {
-      N <- length(w) - 1
-      return(w[1:N])
-    }
-    shortselling.jac <- function(w, ...) {
-      N <- length(w) - 1
-      return(cbind(diag(N), rep(0, N)))
-    }
-  } else {
-    budget <- function(w, ...)
-      return(sum(w) - 1)
-    budget.jac <- function(w, ...) {
-      N <- length(w)
-      return(matrix(1, 1, N))
-    }
-    shortselling <- function(w, ...)
-      return(w)
-    shortselling.jac <- function(w, ...)
-      return(diag(length(w)))
-  }
-  switch(formulation,
-         "rc-double-index" = {
-           R <- R_rc_double_index
-           R_grad <- R_grad_rc_double_index
-         },
-         "rc-over-b-double-index" = {
-           R <- R_rc_over_b_double_index
-           R_grad <- R_grad_rc_over_b_double_index
-         },
-         "rc-over-var vs b" = {
-           R <- R_rc_over_var_vs_b
-           R_grad <- R_grad_rc_over_var_vs_b
-         },
-         "rc-over-var" = {
-           R <- R_rc_over_var
-           R_grad <- R_grad_rc_over_var
-         },
-         "rc-over-sd vs b-times-sd" = {
-           R <- R_rc_over_sd_vs_b_times_sd
-           R_grad <- R_grad_rc_over_sd_vs_b_times_sd
-         },
-         "rc vs b-times-var"  = {
-           R <- R_rc_vs_b_times_var
-           R_grad <- R_grad_rc_vs_b_times_var
-         },
-         "rc vs theta" = {
-           R <- R_rc_vs_theta
-           R_grad <- R_grad_rc_vs_theta
-         },
-         "rc-over-b vs theta" = {
-           R <- R_rc_over_b_vs_theta
-           R_grad <- R_grad_rc_over_b_vs_theta
-         },
-         stop("formulation ", formulation, " is not included.")
-  )
-  if (!use_gradient) R_grad <- NULL
-  has_mu <- !is.null(mu)
-  if (has_mu) {
-    wrap_R <- function(R, lmd_mu, mu, has_theta, N) {
-      if (has_theta) {
-        func <- function(...) {
-          kwargs <- list(...)
-          w_ <- kwargs[[1]]
-          return(R(...) - lmd_mu * t(mu) %*% w_[1:N])
-        }
-      } else {
-        func <- function(...) {
-          kwargs <- list(...)
-          return(R(...) - lmd_mu * t(mu) %*% kwargs[[1]])
-        }
-      }
-      return(func)
-    }
-    wrap_R_grad <- function(R_grad, lmd_mu, mu) {
-      grad <- function(...) {
-        return(R_grad(...) - lmd_mu * mu)
-      }
-      return(grad)
-    }
-    R_ <- wrap_R(R, lmd_mu, mu, has_theta, N)
-    R_grad_ <- wrap_R_grad(R_grad, lmd_mu, mu)
-  } else {
-    R_ <- R
-    R_grad_ <- R_grad
-  }
-  fun_seq <- c(R(w0, Sigma, b))
-  time_seq <- c(0)
-  switch(match.arg(method),
-         "alabama" = {
-           start_time <- proc.time()[3]
-           res <- alabama::constrOptim.nl(w0, R_, R_grad_,
-                                          hin = shortselling,
-                                          hin.jac = shortselling.jac,
-                                          heq = budget,
-                                          heq.jac = budget.jac,
-                                          Sigma = Sigma, b = b,
-                                          control.outer = list(trace = FALSE,
-                                                               itmax = maxiter))
-           end_time <- proc.time()[3]
-         },
-         "slsqp" = {
-           start_time <- proc.time()[3]
-           res <- nloptr::slsqp(w0, R_, R_grad_,
-                                hin = shortselling, hinjac = shortselling.jac,
-                                heq = budget, heqjac = budget.jac,
-                                Sigma = Sigma, b = b,
-                                control = list(xtol_rel = wtol, ftol_rel = ftol))
-           end_time <- proc.time()[3]
-         },
-         stop("method ", method, "is not included.")
-  )
-  # save objective value and elapsed time
-  time_seq <- c(time_seq, end_time - start_time)
-  fun_seq <- c(fun_seq, res$value)
-  w <- res$par
-
-  portfolio_results <- list()
-  portfolio_results$risk_parity <- fun_seq[length(fun_seq)]
-  if (!has_theta) {
-    portfolio_results$w <- w
-    portfolio_results$risk_contribution <- as.vector(w * (Sigma %*% w))
-  } else {
-    portfolio_results$w <- w[1:N]
-    portfolio_results$theta <- w[N+1]
-    portfolio_results$risk_contribution <- as.vector(w[1:N] * (Sigma %*% w[1:N]))
-  }
-  if (has_mu) {
-    portfolio_results$mean_return <- t(mu) %*% portfolio_results$w
-    portfolio_results$risk_parity <- portfolio_results$risk_parity + lmd_mu * portfolio_results$mean_return
-  }
-  portfolio_results$obj_fun <- fun_seq
-  portfolio_results$elapsed_time <- time_seq
-  portfolio_results$convergence <- res$convergence
-  return(portfolio_results)
-}
-
-
 riskParityPortfolioNewton <- function(Sigma, b = rep(1/nrow(Sigma), nrow(Sigma)),
                                       maxiter = 50, ftol = 1e-8) {
   w <- risk_parity_portfolio_nn(Sigma, b, ftol, maxiter)
-  return(list(w = w, risk_contribution = c(w * (Sigma %*% w)),
+  w_Sigmaw <- c(w * (Sigma %*% w))
+  return(list(w = w, relative_risk_contribution = w_Sigmaw / sum(w_Sigmaw),
               obj_fun = obj_function_spinu(Sigma, w, b)))
 }
 
@@ -355,7 +219,8 @@ riskParityPortfolioNewton <- function(Sigma, b = rep(1/nrow(Sigma), nrow(Sigma))
 riskParityPortfolioCyclicalRoncalli <- function(Sigma, b = rep(1/nrow(Sigma), nrow(Sigma)),
                                         maxiter = 50, ftol = 1e-8) {
   w <- risk_parity_portfolio_ccd_roncalli(Sigma, b, ftol, maxiter)
-  return(list(w = w, risk_contribution = c(w * (Sigma %*% w)),
+  w_Sigmaw <- c(w * (Sigma %*% w))
+  return(list(w = w, relative_risk_contribution = w_Sigmaw / sum(w_Sigmaw),
               obj_fun = obj_function_roncalli(Sigma, w, b)))
 }
 
@@ -363,7 +228,8 @@ riskParityPortfolioCyclicalRoncalli <- function(Sigma, b = rep(1/nrow(Sigma), nr
 riskParityPortfolioCyclicalSpinu <- function(Sigma, b = rep(1/nrow(Sigma), nrow(Sigma)),
                                              maxiter = 50, ftol = 1e-8) {
   w <- risk_parity_portfolio_ccd_spinu(Sigma, b, ftol, maxiter)
-  return(list(w = w, risk_contribution = c(w * (Sigma %*% w))))
+  w_Sigmaw <- c(w * (Sigma %*% w))
+  return(list(w = w, relative_risk_contribution = w_Sigmaw / sum(w_Sigmaw)))
 }
 
 # minimize ||w - w0||^2
@@ -383,7 +249,19 @@ projectBudgetLineAndBox <- function(w0, w_lb, w_ub) {
 }
 
 
-#' @title Design of Risk Parity Portfolios
+# minimize ||w - w0||^2
+# s.t.     Cw = c
+#          Dw <= d
+project_onto_eq_and_ineq_constraint_set <- function(w0, Cmat, cvec, Dmat, dvec) {
+  meq <- nrow(Cmat)
+  I <- diag(length(w0))
+  Amat <- -t(rbind(Cmat, Dmat))
+  b0 <- -c(cvec, dvec)
+  return(quadprog::solve.QP(Dmat = I, dvec = w0, Amat = Amat, bvec = b0, meq = meq)$solution)
+}
+
+
+#' @title Design of risk parity portfolios
 #'
 #' @description This function designs risk parity portfolios to equalize/distribute
 #' the risk contributions of the different assets, which is missing if we simply
@@ -395,15 +273,18 @@ projectBudgetLineAndBox <- function(w0, w_lb, w_ub) {
 #' expected return and overall variance. In short, this function solves the
 #' following problem:
 #'
-#'       \code{minimize R(w) - lmd_mu * t(w)\%*\%mu + lmd_var * t(w)\%*\%Sigma\%*\%w}
+#'       \code{minimize    R(w) - lmd_mu * t(w) \%*\% mu + lmd_var * t(w) \%*\% Sigma \%*\% w}
 #'
-#'       \code{subject to sum(w) = 1, w_lb <= w <= w_ub},
+#'       \code{subject to  sum(w) = 1, w_lb <= w <= w_ub},
+#'       \code{            Cmat \%*\% w = cvec, Dmat \%*\% w <= dvec},
 #'
 #' where \code{R(w)} denotes the risk concentration,
-#' \code{t(w)\%*\%mu} is the expected return, \code{t(w)\%*\%Sigma\%*\%w} is the
+#' \code{t(w) \%*\% mu} is the expected return, \code{t(w) \%*\% Sigma \%*\% w} is the
 #' overall variance, \code{lmd_mu} and \code{lmd_var} are the trade-off weights
-#' for the expected return and the variance terms, respectively, and \code{w_lb} and
-#' \code{w_ub} are the lower and upper bound vector values for the portfolio vector \code{w}.
+#' for the expected return and the variance terms, respectively, \code{w_lb} and
+#' \code{w_ub} are the lower and upper bound vector values for the portfolio vector \code{w},
+#' \code{Cmat \%*\% w = cvec} denotes arbitrary linear equality constrains, and
+#' \code{Dmat \%*\% w = dvec} denotes arbitrary linear inequality constrains.
 #'
 #' @details By default, the problem considered is the vanilla risk parity portfolio:
 #' \code{w >= 0, sum(w) = 1}, with no expected return term, and no variance term. In this case,
@@ -432,25 +313,29 @@ projectBudgetLineAndBox <- function(w0, w_lb, w_ub) {
 #'
 #' For more details, please check the vignette.
 #'
-#' @param Sigma covariance or correlation matrix (this is the only mandatory argument)
-#' @param b budget vector, i.e., the risk budgeting targets. The default is the
+#' @param Sigma Covariance or correlation matrix (this is the only mandatory argument).
+#' @param b Budget vector, i.e., the risk budgeting targets. The default is the
 #'        uniform 1/N vector.
-#' @param mu vector of expected returns (only needed if the expected return term
-#'        is desired in the objective)
-#' @param lmd_mu scalar weight to control the importance of the expected return term
-#' @param lmd_var scalar weight to control the importance of the variance term
-#'        (only currently available for the SCA method)
-#' @param w_lb lower bound (either a vector or a scalar) on the value of each
-#'        portfolio weight (only currently available for the SCA method)
-#' @param w_ub upper bound (either a vector or a scalar) on the value of each
-#'        portfolio weight (only currently available for the SCA method)
-#' @param method_init method to compute the vanilla solution. In case of
+#' @param mu Vector of expected returns (only needed if the expected return term
+#'        is desired in the objective).
+#' @param lmd_mu Scalar weight to control the importance of the expected return term.
+#' @param lmd_var Scalar weight to control the importance of the variance term
+#'        (only currently available for the SCA method).
+#' @param w_lb Lower bound (either a vector or a scalar) on the value of each
+#'        portfolio weight.
+#' @param w_ub Upper bound (either a vector or a scalar) on the value of each
+#'        portfolio weight.
+#' @param Cmat Equality constraints matrix.
+#' @param cvec Equality constraints vector.
+#' @param Dmat Inequality constraints matrix.
+#' @param dvec Inequality constraints vector.
+#' @param method_init Method to compute the vanilla solution. In case of
 #'        additional constraints or objective terms, this solution is used as
 #'        the initial point for the subsequent method. The default is
 #'        \code{"cyclical-spinu"}. See details below.
-#' @param method method to solve the non-vanilla formulation. The default is \code{"sca"}.
-#'        See details below.
-#' @param formulation string indicating the risk concentration formulation to be used.
+#' @param method Method to solve the non-vanilla formulation. The default is \code{"sca"}.
+#'        See details below. (DEPRECATED)
+#' @param formulation String indicating the risk concentration formulation to be used.
 #'        It must be one of: "diag", "rc-double-index",
 #'        "rc-over-b-double-index", "rc-over-var vs b",
 #'        "rc-over-var", "rc-over-sd vs b-times-sd",
@@ -461,34 +346,36 @@ projectBudgetLineAndBox <- function(w0, w_lb, w_ub) {
 #'        vanilla risk parity portfolio will be returned. If formulation is
 #'        "diag" then the analytical solution of the risk parity optimization for
 #'        for a diagonal covariance matrix will be returned. See details below.
-#' @param w0 initial value for the portfolio weights. Default is a convex
+#' @param w0 Initial value for the portfolio weights. Default is a convex
 #'        combination of the risk parity portfolio, the (uncorrelated) minimum variance
 #'        portfolio, and the maximum return portfolio.
-#' @param theta0 initial value for theta (in case formulation uses theta). If not provided,
+#' @param theta0 Initial value for theta (in case formulation uses theta). If not provided,
 #'        the optimum solution for a fixed vector of portfolio weights will be used.
-#' @param gamma learning rate for the SCA method
-#' @param zeta factor used to decrease the learning rate at each iteration for the SCA method
-#' @param tau regularization factor
-#' @param maxiter maximum number of iterations for the SCA loop
-#' @param ftol convergence tolerance on the objective function
-#' @param wtol convergence tolerance on the values of the portfolio weights
-#' @param use_gradient this parameter is meaningful only if method is either
+#' @param gamma Learning rate for the SCA method.
+#' @param zeta Factor used to decrease the learning rate at each iteration for the SCA method.
+#' @param tau Regularization factor.
+#' @param maxiter Maximum number of iterations for the SCA loop.
+#' @param ftol Convergence tolerance on the objective function.
+#' @param wtol Convergence tolerance on the values of the portfolio weights.
+#' @param use_gradient This parameter is meaningful only if method is either
 #'        \code{"alabama"} or \code{"slsqp"}. If \code{TRUE} (default value), analytical gradients of the
 #'        objective function will be used (strongly recommended to achieve faster results).
+#' @param use_qp_solver Whether or not to use the general QP solver from
+#'        quadprog to solve each iteration of the SCA algorithm. Default is TRUE.
 #' @return A list containing possibly the following elements:
-#' \item{\code{w}}{optimal portfolio vector}
-#' \item{\code{risk_contribution}}{the risk contribution of every asset}
-#' \item{\code{theta}}{the optimal value for theta (in case that it is part of
-#'                     the chosen formulation)}
-#' \item{\code{obj_fun}}{the sequence of values of the objective function at
-#'                       each iteration}
-#' \item{\code{risk_parity}}{the risk concentration term of the portfolio \code{R(w)}}
-#' \item{\code{mean_return}}{the expected return term of the portoflio \code{t(w)\%*\%mu},
-#'                           if the term is included in the optimization}
-#' \item{\code{variance}}{the variance term of the portfolio \code{t(w)\%*\%Sigma\%*\%w},
-#'                        if the term is included in the optimization}
-#' \item{\code{elapsed_time}}{elapsed time recorded at every iteration}
-#' \item{\code{convergence}}{boolean flag to indicate whether or not the optimization converged}
+#' \item{\code{w}}{Optimal portfolio vector.}
+#' \item{\code{relative_risk_contribution}}{The relative risk contribution of every asset.}
+#' \item{\code{theta}}{Optimal value for theta (in case that it is part of
+#'                     the chosen formulation.)}
+#' \item{\code{obj_fun}}{Sequence of values of the objective function at each iteration.}
+#' \item{\code{risk_concentration}}{Risk concentration term of the portfolio \code{R(w)}.}
+#' \item{\code{mean_return}}{Expected return term of the portoflio \code{t(w)\%*\%mu},
+#'                           if the term is included in the optimization.}
+#' \item{\code{variance}}{Variance term of the portfolio \code{t(w)\%*\%Sigma\%*\%w},
+#'                        if the term is included in the optimization.}
+#' \item{\code{elapsed_time}}{Elapsed time recorded at every iteration.}
+#' \item{\code{convergence}}{Boolean flag to indicate whether or not the optimization converged.}
+#' \item{\code{is_feasible}}{Boolean flag to indicate whether or not the computed portfolio respects the linear constraints.}
 #'
 #' @examples
 #' library(riskParityPortfolio)
@@ -502,16 +389,16 @@ projectBudgetLineAndBox <- function(w0, w_lb, w_ub) {
 #' res <- riskParityPortfolio(Sigma)
 #' names(res)
 #' #> [1] "w"                 "risk_contribution"
+#'
 #' res$w
 #' #> [1] 0.04142886 0.38873465 0.34916787 0.09124019 0.12942842
-#' res$risk_contribution
-#' #> [1] 0.007361995 0.007361995 0.007361995 0.007361995 0.007361995
-#' c(res$w * (Sigma %*% res$w))
-#' #> [1] 0.007361995 0.007361995 0.007361995 0.007361995 0.007361995
+#'
+#' res$relative_risk_contribution
+#' #> [1] 0.2 0.2 0.2 0.2 0.2
 #'
 #' # risk budggeting portfolio
 #' res <- riskParityPortfolio(Sigma, b = c(0.4, 0.4, 0.1, 0.05, 0.05))
-#' res$risk_contribution/sum(res$risk_contribution)
+#' res$relative_risk_contribution
 #' #> [1] 0.40 0.40 0.10 0.05 0.05
 #'
 #' @references
@@ -526,98 +413,161 @@ projectBudgetLineAndBox <- function(w0, w_lb, w_ub) {
 #' risk parity portfolios. <https://arxiv.org/pdf/1311.4057.pdf>
 #'
 #' @author Ze Vinicius and Daniel P. Palomar
+#'
 #' @export
 riskParityPortfolio <- function(Sigma, b = NULL, mu = NULL,
-                                lmd_mu = 1e-4, lmd_var = 0,
+                                lmd_mu = 0, lmd_var = 0,
                                 w_lb = 0, w_ub = 1,
+                                Cmat = NULL, cvec = NULL,
+                                Dmat = NULL, dvec = NULL,
                                 method_init = c("cyclical-spinu", "cyclical-roncalli", "newton"),
                                 method = c("sca", "alabama", "slsqp"),
                                 formulation = NULL, w0 = NULL, theta0 = NULL,
                                 gamma = .9, zeta = 1e-7, tau = NULL,
-                                maxiter = 50, ftol = 1e-8, wtol = 1e-6,
-                                use_gradient = TRUE) {
-  # stocks names
-  stocks_names <- colnames(Sigma)
-  # default values
+                                maxiter = 1000, ftol = 1e-8, wtol = .5e-6,
+                                use_gradient = TRUE, use_qp_solver = TRUE) {
   N <- nrow(Sigma)
-  if (is.null(b)) b <- rep(1/N, N)
+  stocks_names <- colnames(Sigma)
+  # check that constraints are consistent
+  is_Cmat_null <- is.null(Cmat)
+  is_cvec_null <- is.null(cvec)
+  if ((!is_Cmat_null) && is_cvec_null) stop("Matrix Cmat has been given, but vector cvec is NULL.")
+  if (is_Cmat_null && (!is_cvec_null)) stop("Vector cvec has been given, but matrix Cmat is NULL.")
+  if (is_Cmat_null)
+    Cmat <- matrix(1, 1, N)
+  else
+    Cmat <- rbind(Cmat, matrix(1, 1, N))
+  if (is_cvec_null)
+    cvec <- c(1)
+  else
+    cvec <- c(cvec, 1)
+  if (nrow(Cmat) > 1)
+    if (Matrix::rankMatrix(Cmat) != nrow(Cmat)) stop("Cmat contains linearly dependent rows.")
+  if (nrow(Cmat) != length(cvec)) stop("Shapes of Cmat and cvec are inconsistent.")
+  is_Dmat_null <- is.null(Dmat)
+  is_dvec_null <- is.null(dvec)
+  if ((!is_Dmat_null) && is_dvec_null) stop("Matrix Dmat has been given, but vector dvec is NULL.")
+  if (is_Dmat_null && (!is_dvec_null)) stop("Vector dvec has been given, but matrix Dmat is NULL.")
+  # default values
+  if (is.null(b))
+    b <- rep(1/N, N)
+  else if (sum(b) != 1)
+    warning("Budget vector b does not sum up to 1.")
+
   if (length(w_ub) == 1) w_ub <- rep(w_ub, N)
   if (length(w_lb) == 1) w_lb <- rep(w_lb, N)
-  # check problem feasibility
-  if (sum(w_lb) > 1) stop("Problem infeasible: relax the lower bounds.")
-  if (sum(w_ub) < 1) stop("Problem infeasible: relax the upper bounds.")
-  if (length(b) != N) stop("Shape mismatch: b has to have nrow(Sigma) number of elements.")
-
+  has_only_equality_constraints <- all(w_lb == (-Inf), w_ub == Inf)
+  if (!has_only_equality_constraints) {
+    In <- diag(nrow(Sigma))
+    if (is.null(Dmat))
+      Dmat <- rbind(-In, In)
+    else
+      Dmat <- rbind(Dmat, -In, In)
+    if (is.null(dvec))
+      dvec <- c(-w_lb, w_ub)
+    else
+      dvec <- c(dvec, -w_lb, w_ub)
+    if (nrow(Dmat) != length(dvec)) stop("Shapes of Dmat and dvec are inconsistent.")
+  }
+  # define boolean cases for easier reading of code
   has_mu <- !is.null(mu)
-  if (has_mu && (length(mu) != N))
-    stop("Shape mismatch: mu has to have nrow(Sigma) number of elements")
   has_theta <- !is.null(theta0)
   has_var <- lmd_var > 0
   has_formulation <- !is.null(formulation)
   has_fancy_box <- any(w_lb != 0) || any(w_ub != 1)
   has_initial_point <- !is.null(w0)
+  has_equality_constraints <- nrow(Cmat) > 1
+  has_inequality_constraints <- !is_Dmat_null
+  has_std_constraints <- !has_only_equality_constraints && (nrow(Cmat) == 1) && is_Dmat_null
+  is_vanilla_formulation <- !(has_mu || has_theta || has_var || has_fancy_box ||
+                              has_only_equality_constraints || has_equality_constraints ||
+                              has_inequality_constraints)
+
+  # check wrong parameters
+  if (sum(w_lb) > 1) stop("Problem infeasible: relax the lower bounds.")
+  if (sum(w_ub) < 1) stop("Problem infeasible: relax the upper bounds.")
+  if (length(b) != N) stop("Shape mismatch: b has to have nrow(Sigma) number of elements.")
+  if (has_mu && (length(mu) != N))
+    stop("Shape mismatch: mu has to have nrow(Sigma) number of elements")
   if (has_initial_point && (length(w0) != N))
     stop("Shape mismatch: w0 has to have nrow(Sigma) number of elements")
-
-  is_vanilla_formulation <- !(has_mu || has_theta || has_var || has_fancy_box)
+  # if diag, then call diagonal solver
   if (has_formulation && formulation == "diag") {
     if (!is_vanilla_formulation)
-      stop("Additional constraints (box-constraints) or terms (expected return",
-           " or variance) are not supported by the 'diag' formulation.")
+      stop("Additional constraints (box-constraints or other linear constrains) or",
+           " terms (expected return or variance) are not supported by the 'diag' formulation.")
     if (has_initial_point)
       warning("The problem is a naive (diagonal) risk parity portfolio, but an initial",
               " point has been provided: The initial point is being ignored.")
     return(riskParityPortfolioDiagSigma(Sigma, b))
   }
-  if (has_formulation && is_vanilla_formulation)
-      warning("The problem is a vanilla risk parity portofolio, but a nonconvex",
-              " formulation has been chosen. Consider not specifying the formulation",
-              " argument in order to get the guaranteed global solution.")
-  is_vanilla_formulation <- is_vanilla_formulation && !has_formulation
+  # if vanilla, then call the vanilla solver
+  if (is_vanilla_formulation && has_formulation) {
+    warning("The problem is a vanilla risk parity portofolio, but a nonconvex",
+            " formulation has been chosen. Consider not specifying the formulation",
+            " argument in order to use the convex formulation and get a guaranteed",
+            " global solution.")
+    is_vanilla_formulation <- FALSE  # so the nonconvex formulation will be used
+  }
   if (is_vanilla_formulation) {
     if (has_initial_point)
       warning("The problem is a vanilla risk parity portfolio, but an initial",
               " point has been provided: The initial point is being ignored.")
-    switch(match.arg(method_init),
-           "newton" = portfolio <- riskParityPortfolioNewton(Sigma, b, maxiter, ftol),
-           "cyclical-spinu" = portfolio <- riskParityPortfolioCyclicalSpinu(Sigma, b, maxiter, ftol),
-           "cyclical-roncalli" = portfolio <- riskParityPortfolioCyclicalRoncalli(Sigma, b, maxiter, ftol),
-           stop("method_init ", method_init, " is not supported."))
-  } else {
+    portfolio <- switch(match.arg(method_init),
+                        "newton" = riskParityPortfolioNewton(Sigma, b, maxiter, ftol),
+                        "cyclical-spinu" = riskParityPortfolioCyclicalSpinu(Sigma, b, maxiter, ftol),
+                        "cyclical-roncalli" = riskParityPortfolioCyclicalRoncalli(Sigma, b, maxiter, ftol),
+                        stop("method_init ", method_init, " is not supported."))
+  } else {  # nonconvex solver
     if (!has_initial_point) {
-      switch(match.arg(method_init),
-             "newton" = w0 <- riskParityPortfolioNewton(Sigma, b, maxiter, ftol)$w,
-             "cyclical-spinu" = w0 <- riskParityPortfolioCyclicalSpinu(Sigma, b, maxiter, ftol)$w,
-             "cyclical-roncalli" = w0 <- riskParityPortfolioCyclicalRoncalli(Sigma, b, maxiter, ftol)$w,
-             stop("method_init ", method_init, " is not supported."))
+      w_rc <- switch(match.arg(method_init),
+                     "newton" = riskParityPortfolioNewton(Sigma, b, maxiter, ftol)$w,
+                     "cyclical-spinu" = riskParityPortfolioCyclicalSpinu(Sigma, b, maxiter, ftol)$w,
+                     "cyclical-roncalli" = riskParityPortfolioCyclicalRoncalli(Sigma, b, maxiter, ftol)$w,
+                   stop("method_init ", method_init, " is not supported."))
       # create fancy initial point for the case of additional objectives
       w_gmvp <- 1 / diag(Sigma)
       w_gmvp <- w_gmvp / sum(w_gmvp)
-      if(has_mu) {
+      if (has_mu) {
         w_mu <- as.numeric(max(mu) == mu)
         w_mu <- w_mu / sum(w_mu)
       } else w_mu <- 0
       theta_rc <- 1 / (1 + lmd_var + lmd_mu*sum(has_mu))
       theta_mu <- lmd_mu*sum(has_mu) / (1 + lmd_var + lmd_mu*sum(has_mu))
       theta_var <- lmd_var / (1 + lmd_var + lmd_mu*sum(has_mu))
-      w0 <- theta_rc*w0 + theta_mu*w_mu + theta_var*w_gmvp
+      w0 <- theta_rc*w_rc + theta_mu*w_mu + theta_var*w_gmvp
     }
-    # make w0 feasible
-    if (sum(w0) != 1 || any(w0 < w_lb) || any(w0 > w_ub)) {
-      if (has_initial_point) warning("Initial point is infeasible. Projecting it onto the feasible set.")
-      w0 <- projectBudgetLineAndBox(w0, w_lb, w_ub)
+    if (!isFeasiblePortfolio(w0, Cmat, cvec, Dmat, dvec)) {
+      if (has_initial_point) warning("Initial portfolio is unfeasible, projecting it onto the feasible set.")
+      if (has_only_equality_constraints) {
+        w0 <- project_onto_eq_and_ineq_constraint_set(w0 = w0, Cmat = Cmat,
+                                                      cvec = cvec, Dmat = Dmat,
+                                                      dvec = dvec)
+      } else if(has_inequality_constraints) {
+        w0 <- project_onto_eq_and_ineq_constraint_set(w0 = w0, Cmat = Cmat, cvec = cvec,
+                                                      Dmat = Dmat, dvec = dvec)
+      } else {
+        w0 <- projectBudgetLineAndBox(w0 = w0, w_lb = w_lb, w_ub = w_ub)
+      }
     }
-    # solve nonvanilla formulation
+    # solve nonconvex formulation
     switch(match.arg(method),
            "sca" = portfolio <- riskParityPortfolioSCA(Sigma = Sigma, b = b, mu = mu, lmd_mu = lmd_mu,
                                                        lmd_var = lmd_var, w_lb = w_lb, w_ub = w_ub,
-                                                       formulation = formulation, w0 = w0,
-                                                       theta0 = theta0, gamma = gamma, zeta = zeta,
-                                                       tau = tau, maxiter = maxiter, ftol = ftol, wtol = wtol),
+                                                       Cmat = Cmat, cvec = cvec, Dmat = Dmat, dvec = dvec,
+                                                       formulation = formulation, w0 = w0, theta0 = theta0,
+                                                       gamma = gamma, zeta = zeta, tau = tau,
+                                                       maxiter = maxiter, ftol = ftol, wtol = wtol,
+                                                       use_qp_solver = use_qp_solver),
            "slsqp" = ,
            "alabama" = {
+             warning("Methods 'slsqp' and 'alabama' are deprecated and will be removed in the next release.
+                      We strongly recommend the more robust and scalable method = 'sca'.")
+             # reminder: remove argument use_gradient after decrecating...
              if (has_fancy_box) stop("Box constraints are not supported for method ", method)
              if (has_var) stop("Variance term is not supported for method ", method)
+             if (has_only_equality_constraints || has_inequality_constraints)
+               stop("General linear constraints not supported for method ", method)
              portfolio <- riskParityPortfolioGenSolver(Sigma = Sigma, b = b, mu = mu, lmd_mu = lmd_mu,
                                                        formulation = formulation, method = method,
                                                        use_gradient = use_gradient, w0 = w0, theta0 = theta0,
@@ -626,7 +576,7 @@ riskParityPortfolio <- function(Sigma, b = NULL, mu = NULL,
            stop("method ", method, " is not included.")
     )
   }
-  names(portfolio$w) <- stocks_names
-  names(portfolio$risk_contribution) <- stocks_names
+  names(portfolio$w) <- names(portfolio$relative_risk_contribution) <- stocks_names
+  portfolio$is_feasible <- isFeasiblePortfolio(portfolio$w, Cmat, cvec, Dmat, dvec)
   return(portfolio)
 }
